/*
 * ====================================================================
 *
 * MIT License
 * 
 * Copyright (c) [2022] 
 * [Jan Grossniklaus, Jan Kunert, Sascha Hildebrands, Kevin Hinrichs]
 * 
 * Permission is hereby granted, free of charge, to any person
 * obtaining a copy of this software and associated documentation
 * files (the "Software"), to deal in the Software without
 * restriction, including without limitation the rights to use, copy, 
 * modify, merge, publish, distribute, sublicense, and/or sell copies
 * of the Software, and to permit persons to whom the Software is
 * urnished to do so, subject to the following conditions:
 * 
 * The above copyright notice and this permission notice shall
 * be included in all copies or substantial portions of the Software.
 * 
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND,
 * EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES
 * OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND
 * NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS
 * BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN
 * ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN
 * CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
 * SOFTWARE.
 * 
 * ====================================================================
 */

#ifndef WS2812B_H_
#define WS2812B_H_

//*********************************************************************
//
// Data Structures
//
//*********************************************************************
typedef struct
{
    uint8_t sendLedData;
    
    // LED-Pixel Daten
    uint8_t pixelData[464][3]; // 1.329 Bytes SRAM
    
    // DMA Buffer
    uint16_t streamData_PRI[4][(24 * 29) + 1]; // 5.576 Bytes SRAM
    uint16_t streamData_ALT[4][(24 * 29) + 1]; // 5.576 Bytes SRAM
    
}streamDataDataTyp; // 12.481 Bytes SRAM

//*********************************************************************
//
// Prototypes
//
//*********************************************************************
extern void startTimer();
extern void stopTimer();
extern uint8_t sendLED(streamDataDataTyp*);

extern void convertStreamData0PRI1Turn(streamDataDataTyp*);
extern void convertStreamData1PRI1Turn(streamDataDataTyp*);
extern void convertStreamData2PRI1Turn(streamDataDataTyp*);
extern void convertStreamData3PRI1Turn(streamDataDataTyp*);

extern void convertStreamData0ALT2Turn(streamDataDataTyp*);
extern void convertStreamData1ALT2Turn(streamDataDataTyp*);
extern void convertStreamData2ALT2Turn(streamDataDataTyp*);
extern void convertStreamData3ALT2Turn(streamDataDataTyp*);

extern void convertStreamData0PRI3Turn(streamDataDataTyp*);
extern void convertStreamData1PRI3Turn(streamDataDataTyp*);
extern void convertStreamData2PRI3Turn(streamDataDataTyp*);
extern void convertStreamData3PRI3Turn(streamDataDataTyp*);

extern void convertStreamData0ALT4Turn(streamDataDataTyp*);
extern void convertStreamData1ALT4Turn(streamDataDataTyp*);
extern void convertStreamData2ALT4Turn(streamDataDataTyp*);
extern void convertStreamData3ALT4Turn(streamDataDataTyp*);

extern void updatePixelData(streamDataDataTyp*, uint16_t, uint8_t, uint8_t, uint8_t);

extern void startStreamPri(streamDataDataTyp*);
extern void startStreamAlt(streamDataDataTyp*);
extern void startStreamAltReset(streamDataDataTyp*);

//*********************************************************************
//
// Global Variable for LED Stream data for Interrupts
//
//*********************************************************************
EXTERN volatile enum UDMA_TRANSFER_STATE ledStreamTranferState_PRI[4];
EXTERN volatile enum UDMA_TRANSFER_STATE ledStreamTranferState_ALT[4];

EXTERN volatile uint8_t ledResetTimeIsRunnig;

#endif //WS2812B_H_